@@ -238,7 +238,33 @@ class AMPNode:
         Returns styles that need to be appended to the beginning of the
         amp-custom style section.
         """
+        # Create img if necessary
+        if self.tag == 'amp-img' and 'data-hero' in self._other_attrs:
+            self._other_attrs['i-amphtml-ssr'] = None
+            img_attrs = [
+                ('class', 'i-amphtml-fill-content i-amphtml-replaced-content'),
+                ('decoding', 'async'),
+            ]
+
+            attrs_to_copy = [
+                'alt',
+                'attribution',
+                'object-fit',
+                'object-position',
+                'referrerpolicy',
+                'src',
+                'srcset',
+                'sizes',
+                'title',
+            ]
+            for name in [k for k in attrs_to_copy if k in self._other_attrs]:
+                img_attrs.append((name, self._other_attrs[name]))
+
+            self.maybe_img_attrs = img_attrs
+
+        # Translate special attributes to amp-custom CSS
         translation = None
+        did_strip_sizes = False
 
         translations = [k for k in self._other_attrs if k in self.TRANSLATIONS]
         if translations:
@@ -246,15 +272,13 @@ class AMPNode:
 
             css_parts = []
             for t in translations:
-                if t == 'sizes' and 'srcset' not in self._other_attrs:
-                    """According to the Mozilla docs, a sizes attribute without
-                    a valid srcset attribute should have no effect. Therefore,
-                    it should simply be stripped, without producing media
-                    queries.
+                if t == 'sizes' and 'disable-inline-width' in self._other_attrs:
+                    """Sizes is meant to be passed to the img tag for source
+                    selection, but the user didn’t intend for it to have any
+                    side effects. Don’t add any.
 
-                    https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#attr-sizes  # noqa
+                    https://amp.dev/documentation/guides-and-tutorials/learn/amp-html-layout/?format=websites  # noqa
                     """
-
                     continue
 
                 attribute_value = self._other_attrs[t]
@@ -264,6 +288,10 @@ class AMPNode:
                     css_part = Translator.make_translated_css(attribute_value, potential_id)
                 except ValueError:
                     raise self.TransformationError('Invalid value for `{}` attribute'.format(t))
+                else:
+                    if t == 'sizes':
+                        # Need to know so we can apply "responsive" layout
+                        did_strip_sizes = True
 
                 if css_part:
                     css_parts.append(css_part)
@@ -281,6 +309,7 @@ class AMPNode:
                 for t in translations:
                     del self._other_attrs[t]
 
+        # Apply the transformation
         layout_value = self._other_attrs.get('layout')
 
         width = self._parse_length(self._other_attrs.get('width'))
@@ -315,6 +344,8 @@ class AMPNode:
                 layout_value = 'container'
             elif height_is_set and not width_is_set:
                 layout_value = 'fixed_height'
+            elif height_is_set and width_is_set and did_strip_sizes:
+                layout_value = 'responsive'
             else:
                 layout_value = 'fixed'
 
@@ -365,30 +396,6 @@ class AMPNode:
 
         self._other_attrs['i-amphtml-layout'] = layout.value
 
-        # Create img if necessary
-        if self.tag == 'amp-img' and 'data-hero' in self._other_attrs:
-            self._other_attrs['i-amphtml-ssr'] = None
-            img_attrs = [
-                ('class', 'i-amphtml-fill-content i-amphtml-replaced-content'),
-                ('decoding', 'async'),
-            ]
-
-            attrs_to_copy = [
-                'alt',
-                'attribution',
-                'object-fit',
-                'object-position',
-                'referrerpolicy',
-                'src',
-                'srcset',
-                'sizes',
-                'title',
-            ]
-            for name in [k for k in attrs_to_copy if k in self._other_attrs]:
-                img_attrs.append((name, self._other_attrs[name]))
-
-            self.maybe_img_attrs = img_attrs
-
         # Create sizer if necessary
         if all(isinstance(length, self.CSSLength) for length in [width, height]):
             if all([width.numeral != 0, width.unit == height.unit]):
@@ -593,11 +600,6 @@ class AMPRenderer(HTMLParser, object):
                 self._is_expecting_experiment_script = True
 
             amp_element = AMPNode(tag, attrs)
-            if not self._is_test_mode:
-                """AMP Optimizer strips these out, and I believe this is
-                incorrect, because the runtime handles them smartly when
-                left alone."""
-                amp_element.should_strip_translated_attrs = False
 
             try:
                 transformation = amp_element.transform(self._next_auto_id_num)
